pub enum SimulationFailure {
UnsolvableOpcode(usize),
UnsolvableOutput(Witness),
}Expand description
The reason a circuit was deemed unsolvable by the CircuitSimulator.
Variants
UnsolvableOpcode(usize)
The opcode at this index cannot be solved from the witnesses known at that point.
UnsolvableOutput(Witness)
All opcodes can be solved, but this circuit return value is never computed by any of them.