fn fits_in_one_identity<F: AcirField>(
expr: &Expression<F>,
width: usize,
) -> boolExpand description
Checks if this expression can fit into one arithmetic identity
fn fits_in_one_identity<F: AcirField>(
expr: &Expression<F>,
width: usize,
) -> boolChecks if this expression can fit into one arithmetic identity