pub(crate) struct GeneralOptimizer;Expand description
The GeneralOptimizer processes all Expressions to:
- remove any zero-coefficient terms.
- merge any quadratic terms containing the same two witnesses.
This pass does not depend on any other pass and should be the first one in a set of optimizing passes.
Implementations
Sourceimpl GeneralOptimizer
impl GeneralOptimizer
pub(crate) fn optimize<F: AcirField>(opcode: Expression<F>) -> Expression<F>