struct RangeInfo {
switch_points: BTreeSet<usize>,
num_bits: u32,
is_implied: bool,
}Expand description
Information gathered about witnesses which are subject to range constraints.
Fields
switch_points: BTreeSet<usize>Opcode positions which updated this RangeInfo, i.e at which stricter bit size information becomes available.
num_bits: u32Strictest constraint on bit size so far.
is_implied: boolIndicate whether the bit size comes from an assertion or from array indexing, in which cases we can save an equivalent range constraint.