Struct RangeOptimizer

pub(crate) struct RangeOptimizer<'a, F: AcirField> {
    infos: BTreeMap<Witness, RangeInfo>,
    brillig_side_effects: &'a BTreeMap<BrilligFunctionId, bool>,
    circuit: Circuit<F>,
}

Fields

infos: BTreeMap<Witness, RangeInfo>

Maps witnesses to their bit size switch points.

brillig_side_effects: &'a BTreeMap<BrilligFunctionId, bool>

The next potential side effect for each opcode.

circuit: Circuit<F>

Implementations

Source

impl<'a, F: AcirField> RangeOptimizer<'a, F>

pub(crate) fn new( circuit: Circuit<F>, brillig_side_effects: &'a BTreeMap<BrilligFunctionId, bool>, ) -> Self

Creates a new RangeOptimizer by collecting all known range constraints from Circuit.

fn collect_ranges(circuit: &Circuit<F>) -> BTreeMap<Witness, RangeInfo>

Collect range information about witnesses.

pub(crate) fn replace_redundant_ranges( self, order_list: Vec<usize>, ) -> (Circuit<F>, Vec<usize>)

Returns a Circuit where each Witness is only range constrained a minimal number of times that still allows us to avoid executing any new side effects due to their removal.

The idea is to keep only the RANGE opcodes that have strictly smaller bit-size requirements than before, i.e the ones that are at a ‘switch point’. Furthermore, we only keep the switch points that are last before a ‘side-effect’ opcode (i.e a Brillig call). As a result, we simply do a backward pass on the opcodes, so that the last Brillig call is known before reaching a RANGE opcode.