pub(crate) struct UnusedMemoryOptimizer<F: AcirField> {
unused_memory_initializations: HashSet<BlockId>,
circuit: Circuit<F>,
}Expand description
UnusedMemoryOptimizer will remove initializations of memory blocks which are unused.
A first pass collects all memory blocks which are initialized but discards the ones
which are used in a MemoryOp or as input to a BrilligCall.
The second pass removes the opcodes tagged as unused by the first pass.
Fields
unused_memory_initializations: HashSet<BlockId>circuit: Circuit<F>Implementations
Sourceimpl<F: AcirField> UnusedMemoryOptimizer<F>
impl<F: AcirField> UnusedMemoryOptimizer<F>
Sourcepub(crate) fn new(circuit: Circuit<F>) -> Self
pub(crate) fn new(circuit: Circuit<F>) -> Self
Creates a new UnusedMemoryOptimizer by collecting unused memory init
opcodes from Circuit.
Sourcefn collect_unused_memory_initializations(
circuit: &Circuit<F>,
) -> HashSet<BlockId>
fn collect_unused_memory_initializations( circuit: &Circuit<F>, ) -> HashSet<BlockId>
Creates a set of ids for memory blocks for which no [Opcode::MemoryOp]s exist.
These memory blocks can be safely removed.