Function quick_invert

fn quick_invert<F: AcirField>(numerator: F, denominator: F) -> F
A wrapper around field division which skips the inversion if the denominator is ±1.

Field inversion is the most significant cost of solving [Opcode::AssertZero][acir::circuit::opcodes::Opcode::AssertZero] opcodes, which we can avoid when the denominator is ±1.