pub enum ErrorLocation {
Unresolved,
Resolved(OpcodeLocation),
}Expand description
Used by errors to point to a specific opcode as that error’s cause
Some errors don’t have a specific opcode associated with them, or are created without one and added later.
Variants