pub fn insert_value<F: AcirField>(
witness: &Witness,
value_to_insert: F,
initial_witness: &mut WitnessMap<F>,
) -> Result<(), OpcodeResolutionError<F>>Expand description
Inserts value into the initial witness map under the index witness.
Returns an error if there was already a value in the map which does not match the value that one is about to insert