Struct ForeignCallWaitInfo

pub struct ForeignCallWaitInfo<F> {
    pub function: String,
    pub inputs: Vec<ForeignCallParam<F>>,
}
Encapsulates a request from a Brillig VM process that encounters a foreign call opcode where the result of the foreign call has not yet been provided.

The caller must resolve this opcode externally based upon the information in the request.

Fields

function: String

An identifier interpreted by the caller process

inputs: Vec<ForeignCallParam<F>>

Resolved inputs to a foreign call computed in the previous steps of a Brillig VM process