pub(crate) struct GeneralOptimizer;Expand description
The GeneralOptimizer processes all [Expression]s to:
- remove any zero-coefficient terms.
- merge any quadratic terms containing the same two witnesses.
This pass does not depend on any other pass and should be the first one in a set of optimizing passes.
Implementations