pub enum ACVMStatus<F> {
Solved,
InProgress,
Failure(OpcodeResolutionError<F>),
RequiresForeignCall(ForeignCallWaitInfo<F>),
RequiresAcirCall(AcirCallWaitInfo<F>),
}Variants
Solved
All witnesses have been computed and all opcodes have been successfully resolved. Execution is complete.
InProgress
The ACVM is processing the circuit, i.e solving the opcodes. This status is used to resume execution after it has been paused.
Failure(OpcodeResolutionError<F>)
The ACVM has encountered an irrecoverable error while executing the circuit and can not progress. Most commonly this will be due to an unsatisfied constraint due to invalid inputs to the circuit.
RequiresForeignCall(ForeignCallWaitInfo<F>)
The ACVM has encountered a request for a Brillig [foreign call][brillig_vm::brillig::Opcode::ForeignCall]
to retrieve information from outside of the ACVM. The result of the foreign call must be passed back
to the ACVM using ACVM::resolve_pending_foreign_call.
Once this is done, the ACVM can be restarted to solve the remaining opcodes.
RequiresAcirCall(AcirCallWaitInfo<F>)
The ACVM has encountered a request for an ACIR [call][acir::circuit::Opcode] to execute a separate ACVM instance. The result of the ACIR call must be passed back to the ACVM.
Once this is done, the ACVM can be restarted to solve the remaining opcodes.