pub struct AcirCallWaitInfo<F> {
pub id: AcirFunctionId,
pub initial_witness: WitnessMap<F>,
}Expand description
Encapsulates a request from the ACVM that encounters an ACIR call opcode where the result of the circuit execution has not yet been provided.
The caller must resolve this opcode externally based upon the information in the request.
Fields
id: AcirFunctionIdIndex in the list of ACIR function’s that should be called
initial_witness: WitnessMap<F>Initial witness for the given circuit to be called